;Copyright (C) 2004-2011 John T. Haller of PortableApps.com
;Copyright (C) 2006-2007 Geoff Shearsmith
;Copyright (C) 2008 OliverK
;Copyright (C) 2011 JW Hough

;Website: http://PortableApps.com/CamStudioPortable

;This software wrapper is OSI Certified Open Source Software.
;OSI Certified is a certification mark of the Open Source Initiative.

;This program wrapper is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.

;This program wrapper is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.

;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
!define PORTABLEAPPNAME "CamStudio Portable"
!define APPNAME "CamStudio"
!define NAME "CamStudioPortable"
!define VER "2.0.0.0"
!define WEBSITE "PortableApps.com\CamStudioPortable"
!define DEFAULTEXE "Recorder.exe"
!define DEFAULTAPPDIR "CamStudio"
!define DEFAULTSETTINGSDIR "settings"
!define LAUNCHERLANGUAGE "English"
!define OTHEREXE1 "Producer.exe"
!define OTHEREXE2 "Playplus.exe"
!define OTHEREXE3 "Recorder.exe"

;=== Program Details
Name "${PORTABLEAPPNAME}"
OutFile "..\..\${NAME}.exe"
Caption "${PORTABLEAPPNAME} | PortableApps.com"
VIProductVersion "${VER}"
VIAddVersionKey ProductName "${PORTABLEAPPNAME}"
VIAddVersionKey Comments "Allows ${APPNAME} to be run from a removable drive.  For additional details, visit ${WEBSITE}"
VIAddVersionKey CompanyName "PortableApps.com"
VIAddVersionKey LegalCopyright "PortableApps.com & Contributors"
VIAddVersionKey FileDescription "${PORTABLEAPPNAME}"
VIAddVersionKey FileVersion "${VER}"
VIAddVersionKey ProductVersion "${VER}"
VIAddVersionKey InternalName "${PORTABLEAPPNAME}"
VIAddVersionKey LegalTrademarks "PortableApps.com is a Trademark of Rare Ideas, LLC."
VIAddVersionKey OriginalFilename "${NAME}.exe"

;=== Runtime Switches
CRCCheck on
WindowIcon off
SilentInstall silent
AutoCloseWindow true
RequestExecutionLevel user
XPStyle on

; Best Compression
SetCompress auto
SetCompressor /SOLID lzma
SetCompressorDictSize 32
SetDatablockOptimize on

;=== Include
!include ReadINIStrWithDefault.nsh
!include Registry.nsh
!include FileFunc.nsh
!include CheckForPlatformSplashDisable.nsh

;=== Program Icon
Icon "..\..\App\AppInfo\appicon.ico"

;=== Languages
LoadLanguageFile "${NSISDIR}\Contrib\Language files\${LAUNCHERLANGUAGE}.nlf"
!include PortableApps.comLauncherLANG_${LAUNCHERLANGUAGE}.nsh

;===Variables
Var PROGRAMDIRECTORY
Var SETTINGSDIRECTORY
Var ADDITIONALPARAMETERS
Var EXECSTRING
Var PROGRAMEXECUTABLE
Var DISABLESPLASHSCREEN
Var MISSINGFILEORPATH

Section "Main"
	;=== Find the INI file, if there is one
		IfFileExists "$EXEDIR\${NAME}.ini" "" NoINI
			;=== Read the parameters from the INI file
			${ReadINIStrWithDefault} $0 "$EXEDIR\${NAME}.ini" "${NAME}" "${APPNAME}Directory" "App\${DEFAULTAPPDIR}"
			StrCpy $PROGRAMDIRECTORY "$EXEDIR\$0"
			${ReadINIStrWithDefault} $0 "$EXEDIR\${NAME}.ini" "${NAME}" "SettingsDirectory" "Data\${DEFAULTSETTINGSDIR}"
			StrCpy $SETTINGSDIRECTORY "$EXEDIR\$0"
			${ReadINIStrWithDefault} $ADDITIONALPARAMETERS "$EXEDIR\${NAME}.ini" "${NAME}" "AdditionalParameters" ""
			${ReadINIStrWithDefault} $PROGRAMEXECUTABLE "$EXEDIR\${NAME}.ini" "${NAME}" "${APPNAME}Executable" "${DEFAULTEXE}"
			${ReadINIStrWithDefault} $DISABLESPLASHSCREEN "$EXEDIR\${NAME}.ini" "${NAME}" "DisableSplashScreen" "false"

			StrCmp $PROGRAMDIRECTORY "$EXEDIR\App\${DEFAULTAPPDIR}" "" CheckForFile
			StrCmp $SETTINGSDIRECTORY "$EXEDIR\Data\${DEFAULTSETTINGSDIR}" "" CheckForFile

		CheckForFile:
			IfFileExists "$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" FoundProgramEXE NoProgramEXE

	NoINI:
		;=== No INI file, so we'll use the defaults
		StrCpy $ADDITIONALPARAMETERS ""
		StrCpy $PROGRAMEXECUTABLE "${DEFAULTEXE}"
		StrCpy $DISABLESPLASHSCREEN "false"

		IfFileExists "$EXEDIR\App\${DEFAULTAPPDIR}\${DEFAULTEXE}" "" NoProgramEXE
			StrCpy $PROGRAMDIRECTORY "$EXEDIR\App\${DEFAULTAPPDIR}"
			StrCpy $SETTINGSDIRECTORY "$EXEDIR\Data\${DEFAULTSETTINGSDIR}"
			GoTo FoundProgramEXE

		NoProgramExe:
		;=== Program executable not where expected
		StrCpy $MISSINGFILEORPATH $PROGRAMEXECUTABLE
		MessageBox MB_OK|MB_ICONEXCLAMATION `$(LauncherFileNotFound)`
		Abort

      FoundProgramEXE:
		;=== Check if running
		FindProcDLL::FindProc "${DEFAULTEXE}"
        	StrCmp $R0 "1" WarnAnotherInstance DisplaySplash

	WarnAnotherInstance:
		MessageBox MB_OK|MB_ICONINFORMATION `$(LauncherAlreadyRunning)`
		Abort

	DisplaySplash:
		${CheckForPlatformSplashDisable} $DISABLESPLASHSCREEN
		StrCmp $DISABLESPLASHSCREEN "true" GetPassedParameters
		;=== Show the splash screen while processing registry entries
		InitPluginsDir
		File /oname=$PLUGINSDIR\splash.jpg "CamStudioPortable.jpg"
		newadvsplash::show /NOUNLOAD 1200 0 0 -1 /L $PLUGINSDIR\splash.jpg

	GetPassedParameters:
		;=== Get any passed parameters
		${GetParameters} $0
		StrCmp "'$0'" "''" "" LaunchProgramParameters

		;=== No parameters
		StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE"`
		Goto AdditionalParameters

	LaunchProgramParameters:
		StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" $0`

	AdditionalParameters:
		StrCmp $ADDITIONALPARAMETERS "" SettingsDirectory

		;=== Additional Parameters
		StrCpy $EXECSTRING `$EXECSTRING $ADDITIONALPARAMETERS`

	SettingsDirectory:
		;=== Set the settings directory if we have a path
		IfFileExists "$SETTINGSDIRECTORY\*.*" RegistryBackup
			CreateDirectory $SETTINGSDIRECTORY
			CopyFiles "$EXEDIR\App\DefualtData\*.*" $SETTINGSDIRECTORY

	RegistryBackup:
		;=== Backup the registry
		${registry::KeyExists} "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.avi\OpenWithList" $R0
			StrCmp $R0 "0" "" +2
			${registry::SaveKey} "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.avi\OpenWithList" "$APPDATA\CamStudioPortableBackup.reg" "" $0
		${registry::KeyExists} "HKEY_CURRENT_USER\Software\Local AppWizard-Generated Applications-BackupByCamStudioPortable" $R0
		StrCmp $R0 "0" RegistryWrite
		${registry::KeyExists} "HKEY_CURRENT_USER\Software\Local AppWizard-Generated Applications" $R0
		StrCmp $R0 "-1" RegistryWrite
		${registry::MoveKey} "HKEY_CURRENT_USER\Software\Local AppWizard-Generated Applications" "HKEY_CURRENT_USER\Software\Local AppWizard-Generated Applications-BackupByCamStudioPortable" $R0
		
	RegistryWrite:
		${registry::RestoreKey} "$SETTINGSDIRECTORY\CamStudioPortable.reg" $R0
		Sleep 2000
		StrCmp $R0 '0' LaunchNow ;successfully restored key

	LaunchNow:
	Rename "$SETTINGSDIRECTORY\CamStudio.ini" "$PROGRAMDIRECTORY\CamStudio.ini"
	Rename "$SETTINGSDIRECTORY\CamShapes.ini" "$PROGRAMDIRECTORY\CamShapes.ini"
	Rename "$SETTINGSDIRECTORY\CamLayout.ini" "$PROGRAMDIRECTORY\CamLayout.ini"
	Rename "$SETTINGSDIRECTORY\CamData.ini" "$PROGRAMDIRECTORY\CamData.ini"
	Rename "$SETTINGSDIRECTORY\CamStudio.Producer.Data.ini" "$PROGRAMDIRECTORY\CamStudio.Producer.Data.ini"
	Rename "$SETTINGSDIRECTORY\CamStudio.Producer.ini" "$PROGRAMDIRECTORY\CamStudio.Producer.ini"

		ExecWait $EXECSTRING
		GoTo CheckRunning

	CheckRunning:
		Sleep 1000
		FindProcDLL::FindProc "${DEFAULTEXE}"
		    StrCmp $R0 "1" CheckRunning
		FindProcDLL::FindProc "${OTHEREXE1}"
		    StrCmp $R0 "1" CheckRunning
		FindProcDLL::FindProc "${OTHEREXE2}"
		    StrCmp $R0 "1" CheckRunning
	    FindProcDLL::FindProc "${OTHEREXE3}"
            StrCmp $R0 "1" CheckRunning

	Rename "$PROGRAMDIRECTORY\CamStudio.ini" "$SETTINGSDIRECTORY\CamStudio.ini"
	Rename "$PROGRAMDIRECTORY\CamShapes.ini" "$SETTINGSDIRECTORY\CamShapes.ini"
	Rename "$PROGRAMDIRECTORY\CamLayout.ini" "$SETTINGSDIRECTORY\CamLayout.ini"
	Rename "$PROGRAMDIRECTORY\CamData.ini" "$SETTINGSDIRECTORY\CamData.ini"
	Rename "$PROGRAMDIRECTORY\CamStudio.Producer.Data.ini" "$SETTINGSDIRECTORY\CamStudio.Producer.Data.ini"
	Rename "$PROGRAMDIRECTORY\CamStudio.Producer.ini" "$SETTINGSDIRECTORY\CamStudio.Producer.ini"
	
		${registry::DeleteKey} "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.avi\OpenWithList" $R0
		IfFileExists "$APPDATA\CamStudioPortableBackup.reg" "" +3
			${registry::RestoreKey} "$APPDATA\CamStudioPortableBackup.reg" $R0
			Delete "$APPDATA\CamStudioPortableBackup.reg"
		CreateDirectory $SETTINGSDIRECTORY
			${registry::SaveKey} "HKEY_CURRENT_USER\Software\Local AppWizard-Generated Applications" "$SETTINGSDIRECTORY\CamStudioPortable.reg" "" $0
			Sleep 100
		${registry::DeleteKey} "HKEY_CURRENT_USER\Software\Local AppWizard-Generated Applications" $R0
		${registry::KeyExists} "HKEY_CURRENT_USER\Software\Local AppWizard-Generated Applications-BackupByCamStudioPortable" $R0
		StrCmp $R0 "-1" TheEnd
		${registry::MoveKey} "HKEY_CURRENT_USER\Software\Local AppWizard-Generated Applications-BackupByCamStudioPortable" "HKEY_CURRENT_USER\Software\Local AppWizard-Generated Applications" $R0
		
	TheEnd:
		${registry::Unload}
		newadvsplash::stop /WAIT
SectionEnd